package com.student.snowglobe;

import java.util.List;
import java.util.Random;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.res.Configuration;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.view.Display;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnTouchListener;
import android.view.animation.Animation;
import android.view.animation.AnimationSet;
import android.view.animation.TranslateAnimation;
import android.widget.ImageView;
import android.widget.LinearLayout;

/**
   A snow globe class
   CS185c Homework 4
   @author SJSU Student
   @version 1.0 2010/11/24
 */
public class SnowGlobe extends Activity implements SensorEventListener, OnTouchListener
{
   /** 
      Called when the activity is first created
      @param savedInstanceState
    */
   @Override
   public void onCreate(Bundle savedInstanceState)
   {
      super.onCreate(savedInstanceState);
      setContentView(R.layout.main);
      
      snowGlobe = (LinearLayout) findViewById(R.id.snow_globe);
      snowGlobe.setOnTouchListener(this);
      
      // Changes the background snow globe image depending on the orientation
      if(getResources().getConfiguration().orientation == Configuration.ORIENTATION_PORTRAIT)
      {
         snowGlobe.setBackgroundResource(R.drawable.globe_portrait);
      }
      else
      {
         snowGlobe.setBackgroundResource(R.drawable.globe_landscape);
      }

      // Creates the LocationManager and LocationListener for the GPS location
      LocationManager locationManager = (LocationManager) this.getSystemService(Context.LOCATION_SERVICE);
      LocationListener locationListener = new LocationListener()
      {
         public void onLocationChanged(Location location)
         {
            coordinates = location;
         }

         public void onStatusChanged(String provider, int status, Bundle extras) {}

         public void onProviderEnabled(String provider) {}

         public void onProviderDisabled(String provider) {}
      };
      locationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER, 0, 0, locationListener);
      
      // Creates the SensorManager and gets a Sensor object for the accelerometer
      manager = (SensorManager)getSystemService(SENSOR_SERVICE);
      List<Sensor> list = manager.getSensorList(Sensor.TYPE_ACCELEROMETER);
      if (list != null && list.size() > 0)
      {
          accelerometer = list.get(0);
      }
      
      // Gets the width and height of the display
      Display display = getWindowManager().getDefaultDisplay(); 
      width = display.getWidth();
      height = display.getHeight();
      
      // Creates the snowflakes
      createSnow();
   }

   /**
      Registers the listener for SensorEvents from the accerometer
    */
   @Override
   public void onResume()
   {
      super.onResume();
      if (accelerometer != null)
      {
         manager.registerListener(this, accelerometer, SensorManager.SENSOR_DELAY_NORMAL);
      }
   }

   /**
      Removes the listener for SensorEvents from the accerometer
    */
   @Override
   public void onPause()
   {
      super.onPause();
      if (accelerometer != null)
      {
         manager.unregisterListener(this);
      }
   }

   /**
      Displays an alert with the current GPS coordinates
    */
   private void showLocation()
   {
      double latitude;
      double longitude;
      try
      {
         latitude = coordinates.getLatitude();
         longitude = coordinates.getLongitude();
      }
      catch (Exception e)
      {
         latitude = 0;
         longitude = 0;
      }
      
      AlertDialog.Builder builder = new AlertDialog.Builder(this);
      builder.setMessage("Latitude:\n" + latitude + "\nLongitude:\n" + longitude).setCancelable(true);
      alertDialog = builder.create();
      alertDialog.setButton("OK", new DialogInterface.OnClickListener()
      {
         public void onClick(DialogInterface dialog, int which)
         {
            return;
         } 
      });
      alertDialog.show();
   }
   
   /**
      Creates ImageView objects using a snowflake PNG image with random sizes,
      sets the visibility to invisible, and then adds them to the parent view
    */
   private void createSnow()
   {
      // Initializes the last sensor update time and coordinates to zero
      lastUpdate = 0;
      lastSensorX = 0;
      lastSensorY = 0;
      lastSensorZ = 0;
      
      snowflakes = new ImageView[NUM_SNOWFLAKES];
      random = new Random();
      for (int i = 0; i < snowflakes.length; i++)
      {
         snowflakes[i] = new ImageView(this);
         snowflakes[i].setImageResource(R.drawable.snow);
         snowflakes[i].setAdjustViewBounds(true);
         snowflakes[i].setVisibility(View.GONE);
         snowflakes[i].setFocusableInTouchMode(false);
         snowflakes[i].setId(i);
         
         int size = random.nextInt(MAX_SIZE);
         snowGlobe.addView(snowflakes[i], size, size);
      }
   }
   
   /**
      Starts the snow by using the created snowflakes and TranslateAnimation
      class to animate the snow fall
      @param direction the direction of the snow fall
    */
   private void startSnow(String direction)
   {
      for (ImageView snowflake : snowflakes)
      {
         set = new AnimationSet(true);
         set.setFillAfter(true);
         
         float position = random.nextFloat();
         preTranslate = new TranslateAnimation(
               Animation.RELATIVE_TO_PARENT, position, 
               Animation.RELATIVE_TO_PARENT, position,
               Animation.RELATIVE_TO_PARENT, 0,
               Animation.RELATIVE_TO_PARENT, 0);
         set.addAnimation(preTranslate);
         
         if (direction.equals("shake"))
         {
            postTranslate = new TranslateAnimation(
                  Animation.RELATIVE_TO_PARENT, 0,
                  Animation.RELATIVE_TO_PARENT, 0,
                  Animation.RELATIVE_TO_PARENT, -1,
                  Animation.RELATIVE_TO_PARENT, 1);
            postTranslate.setDuration(DURATION_TIME);
            set.addAnimation(postTranslate);
         }
         else if (direction.equals("swipe"))
         {
            int fromX = 0;
            int toX = 0;           
            if (firstX - lastX > 0 && firstX - lastX > SWIPE_MIN_DISTANCE)
            {
               fromX = 1;
               toX = -1;
            }
            else if (lastX - firstX > 0 && lastX - firstX > SWIPE_MIN_DISTANCE)
            {
               fromX = -1;
               toX = 1;
            }
            postTranslate = new TranslateAnimation(
                  Animation.RELATIVE_TO_PARENT, fromX,
                  Animation.RELATIVE_TO_PARENT, toX,
                  Animation.RELATIVE_TO_PARENT, 0,
                  Animation.RELATIVE_TO_PARENT, 0);
            postTranslate.setDuration(DURATION_TIME);
            set.addAnimation(postTranslate);
            
            int fromY = 0;
            int toY = 0;
            if (firstY - lastY > 0 && firstY - lastY > SWIPE_MIN_DISTANCE)
            {
               fromY = 1;
               toY = -1;
            }
            else if (lastY - firstY > 0 && lastY - firstY > SWIPE_MIN_DISTANCE)
            {
               fromY = -1;
               toY = 1;
            }
            postTranslate = new TranslateAnimation(
                  Animation.RELATIVE_TO_PARENT, 0,
                  Animation.RELATIVE_TO_PARENT, 0,
                  Animation.RELATIVE_TO_PARENT, fromY,
                  Animation.RELATIVE_TO_PARENT, toY);
            postTranslate.setDuration(DURATION_TIME);
            set.addAnimation(postTranslate);
         }

         ImageView temp = (ImageView) snowGlobe.findViewById(snowflake.getId());
         temp.setVisibility(View.VISIBLE);
         temp.startAnimation(set);
      }
   }
   
   /**
      Called when sensor values have changed; detects the "shake" that starts 
      the snow
      @param event the SensorEvent
    */
   public void onSensorChanged(SensorEvent event)
   {
      long currentTime = System.currentTimeMillis();
      if ((currentTime - lastUpdate) > 200)
      {
         long diffTime = (currentTime - lastUpdate);
         lastUpdate = currentTime;

         float x = event.values[0];
         float y = event.values[1];
         float z = event.values[2];

         float force = Math.abs(x + y + z - lastSensorX - lastSensorY - lastSensorZ) / diffTime * 10000;
         if (force > FORCE_THRESHOLD)
         {
            startSnow("shake");
         }

         lastSensorX = x;
         lastSensorY = y;
         lastSensorZ = z;
      }
   }
   
   /**
      Called when the accuracy of a sensor has changed
      @param sensor the Sensor
      @param accuracy the new accuracy of this sensor 
    */
   public void onAccuracyChanged(Sensor sensor, int accuracy) {}

   /**
      Called when a touch event is dispatched to a view; handles the swipe that
      starts the snow
      @param v the view the touch event has been dispatched to
      @param event the MotionEvent object containing full information about the event
      @return true if the listener has consumed the event, false otherwise
    */
   public boolean onTouch(View v, MotionEvent event)
   {
      if (event.getAction() == MotionEvent.ACTION_DOWN)
      {
         firstX = event.getX();
         firstY = event.getY();

         return true;
      }
      else if (event.getAction() == MotionEvent.ACTION_UP)
      {
         lastX = event.getX();
         lastY = event.getY();

         int distance = (int) Math.sqrt(Math.pow(lastX - firstX, 2) + Math.pow(lastY - firstY, 2));
         
         if (distance > SWIPE_MIN_DISTANCE)
         {            
            startSnow("swipe");
         }
         else
         {
            showLocation();
         }
         return true;
      }
      return false;
   }
   
   int width;
   int height;
   private float firstX;
   private float firstY;
   private float lastX;
   private float lastY;
   private SensorManager manager;
   private Sensor accelerometer;
   private long lastUpdate;
   private float lastSensorX;
   private float lastSensorY;
   private float lastSensorZ;
   private LinearLayout snowGlobe;
   private Location coordinates;
   private AlertDialog alertDialog;
   private ImageView[] snowflakes;
   private AnimationSet set;
   private Animation preTranslate;
   private Animation postTranslate;
   private Random random;

   private static final int FORCE_THRESHOLD = 800;
   private static final int NUM_SNOWFLAKES = 50;
   private static final int MAX_SIZE = 32;
   private static final int DURATION_TIME = 10000;
   private static final int SWIPE_MIN_DISTANCE = 120;
}
